//
// TBMK.h, v1.0, Thu Dec  6 14:17:04 CET 2007
// Author: P.Konczykowski
//

#ifndef __TBMK__
#define __TBMK__

#include "TObject.h"
#include "TLorentzVector.h"

////////////////////////////////////////////////////////////////////////////////
//
// TBMK.h
//
// BMK class
// 
////////////////////////////////////////////////////////////////////////////////

class TBMK : public TObject
{
  protected :

    // Kinematical Variables

    Double_t xb;
    Double_t Q2;
    Double_t t;        // = Delta^2
    Double_t phi_bel;
    Double_t varphi;   // As defined in BMK article

    Double_t Eb;       // Beam energy
    Double_t M;        //Proton mass

    Double_t eps;
    Double_t K2;
    Double_t t1;
    Double_t t0;
    Double_t y;

    Double_t thph;

    // 3-vectors

    TVector3 b;

    // 4-vectors

    TLorentzVector k;  // As defined in BMK article: e(k)h(p1)->e(kp)h(p2)gamma(q2)
    TLorentzVector p1;
    TLorentzVector kp;
    TLorentzVector p2;
    TLorentzVector q2;

    TLorentzVector q1; // Virtual photon 4-vector

    // BH propagators

    Double_t prop1;
    Double_t prop2;

    //Coef for the amplitudes
    Double_t alpha;
    Double_t alpha_I[4];
    Double_t alpha_DVCS;
    Double_t cunp_bh[3];
    Double_t C_dvcs_unp;
    Double_t C_I_unp_Re;
    Double_t C_I_unp_Im;
    Double_t dC_I_unp_Re;

    Double_t c_L_bh[2];
    Double_t alpha_DVCS_L;
    Double_t C_dvcs_L;
    Double_t alpha_I_L[4];
    Double_t C_I_L_Re;
    Double_t C_I_L_Im;
    Double_t dC_I_L_Re;

    Double_t c_T_bh[3];
    Double_t alpha_DVCS_T[2];
    Double_t C_dvcs_T[2];
    Double_t alpha_I_T[8];
    Double_t C_I_Tp_Re;
    Double_t C_I_Tp_Im;
    Double_t C_I_Tm_Re;
    Double_t C_I_Tm_Im;
    Double_t dC_I_Tp_Re;
    Double_t dC_I_Tm_Im;

    // Amplitudes
    Double_t TBH2;
    Double_t TDVCS2;
    Double_t TI2;

    Double_t TBH2_L;
    Double_t TDVCS2_L;
    Double_t TI2_L;

    Double_t TBH2_T;
    Double_t TDVCS2_T;
    Double_t TI2_T;

    //CrossSection
    Double_t sigma;

    // Flags

    Bool_t kin_init;

    Bool_t BHUnpC_init;
    Bool_t DVCSUnpC_init;
    Bool_t IUnpC_init;

    Bool_t BH_LC_init;
    Bool_t DVCS_LC_init;
    Bool_t I_LC_init;

    Bool_t BH_TC_init;
    Bool_t DVCS_TC_init;
    Bool_t I_TC_init;

    //GPDs H and E calculated by the Guzey model
    Double_t H_Im;
    Double_t H_Re;
    Double_t E_Im;
    Double_t E_Re;

    //GPDs Ht and Et
    Double_t Ht_Im;
    Double_t Ht_Re;
    Double_t Et_Im;
    Double_t Et_Re;

  public :

    TBMK();
    TBMK(Double_t Ebeam);
    virtual ~TBMK();

    void Init(Double_t Ebeam);

    void Kin(Double_t xh, Double_t Q2h, Double_t th);

    void BHUnpC();

    void DVCSUnpC();

    void IUnpC(Int_t he);

    void BH_LC(Int_t he,Int_t hp);

    void DVCS_LC(Int_t he,Int_t hp);

    void I_LC(Int_t he,Int_t hp);

    void BH_TC(Int_t he);

    void DVCS_TC(Int_t he);

    void I_TC(Int_t he);

    void GetSigma(Double_t phih,Int_t he,Double_t *sig);
    /*   sig[0]=GetBH2(phi_exp); */
    /*   sig[1]=GetI2(phi_exp,he); */
    /*   sig[2]=GetDVCS2(phi_exp); */
    /*   sig[3]=sig[0]+sig[1]+sig[2]; */

    void GetSigmaL(Double_t phih,Int_t he,Int_t hp,Double_t *sig);
    /*   sig[0]=GetBH2(phi_exp)+GetBH2_L(phi_exp,he,hp); */
    /*   sig[1]=GetI2(phi_exp,he)+GetI2_L(phi_exp,he,hp); */
    /*   sig[2]=GetDVCS2(phi_exp)+GetDVCS2_L(phi_exp,he,hp); */
    /*   sig[3]=sig[0]+sig[1]+sig[2]; */

    void GetSigmaT(Double_t phih,Int_t he,Double_t *sig);
    /*   sig[0]=GetBH2(phi_exp)+GetBH2_T(phi_exp,he); */
    /*   sig[1]=GetI2(phi_exp,he)+GetI2_T(phi_exp,he); */
    /*   sig[2]=GetDVCS2(phi_exp)+GetDVCS2_T(phi_exp,he); */
    /*   sig[3]=sig[0]+sig[1]+sig[2]; */

    Double_t GetBH2(Double_t phih);

    Double_t GetDVCS2(Double_t phih);

    Double_t GetI2(Double_t phih,Int_t he);

    Double_t GetBH2_L(Double_t phih,Int_t he,Int_t hp);

    Double_t GetDVCS2_L(Double_t phih,Int_t he,Int_t hp);

    Double_t GetI2_L(Double_t phih,Int_t he,Int_t hp);

    Double_t GetBH2_T(Double_t phih,Int_t he);

    Double_t GetDVCS2_T(Double_t phih,Int_t he);

    Double_t GetI2_T(Double_t phih,Int_t he);

    Double_t GMp(Double_t q2) { return 2.79285/TMath::Power(1.-q2/TMath::Power(0.843,2.),2.) ;}

    Double_t GMn(Double_t q2) { return -1.91/TMath::Power(1.-q2/TMath::Power(0.843,2.),2.) ;}

    Double_t GEp(Double_t q2) { return 1./TMath::Power(1.-q2/TMath::Power(0.843,2.),2.) ; }

    Double_t GEn(Double_t q2) { return -1.*(-1.25)*(-1.91)*q2/4./TMath::Power(0.938271998,2.)
				  /(1.-18.3*q2/4./TMath::Power(0.938271998,2.))
				  /TMath::Power(1.-q2/TMath::Power(0.843,2.),2.);}

    Double_t F1p(Double_t q2) { return ((-1.)*q2/4./TMath::Power(0.938271998,2.)*GMp(q2)+GEp(q2))
				  /(1.-q2/4./TMath::Power(0.938271998,2.));}

    Double_t F2p(Double_t q2) { return (GMp(q2)-GEp(q2))/(1.-q2/4./TMath::Power(0.938271998,2.));}

    Double_t F1n(Double_t q2) { return ((-1.)*q2/4./TMath::Power(0.938271998,2.)*
					GMn(q2)+GEn(q2))/(1.-q2/4./TMath::Power(0.938271998,2.));}

    Double_t F2n(Double_t q2) { return (GMn(q2)-GEn(q2))/(1.-q2/4./TMath::Power(0.938271998,2.)); }

    ClassDef(TBMK,1) // BMK Base Class
}; // End of TBMK class definition

#endif
